@@ -32,7 +32,7 @@
 #include <gtk/gtkbutton.h>
 #include <gtk/gtkwindow.h>
 #include <gdk/gdk.h>
-
+#include <gdk/gdkkeysyms.h>
 
 #define HILDON_DESKTOP_POPUP_MENU_GET_PRIVATE(object) \
 	        (G_TYPE_INSTANCE_GET_PRIVATE ((object), HILDON_DESKTOP_TYPE_POPUP_MENU, HildonDesktopPopupMenuPrivate))
@@ -41,12 +41,14 @@ G_DEFINE_TYPE (HildonDesktopPopupMenu, hildon_desktop_popup_menu, GTK_TYPE_VBOX)
 
 enum
 {
-  PROP_POPUP_ITEM_HEIGHT=1
+  PROP_POPUP_ITEM_HEIGHT=1,
+  PROP_POPUP_RESIZE_PARENT	  
 };
 
 enum
 {
   SIGNAL_POPUP_RESIZE,
+  SIGNAL_POPUP_SHOW_CONTROLS,
   N_SIGNALS
 };
 
@@ -54,21 +56,23 @@ static gint signals[N_SIGNALS];
 
 struct _HildonDesktopPopupMenuPrivate
 {
-  GtkWidget *viewport;
-  GtkWidget *box_items;
+  GtkWidget    *viewport;
+  GtkWidget    *box_items;
 
-  GtkWidget *box_buttons;
-  GtkWidget *scroll_down;
-  GtkWidget *scroll_up;
-  gboolean   controls_on;
+  GtkWidget    *box_buttons;
+  GtkWidget    *scroll_down;
+  GtkWidget    *scroll_up;
+  gboolean      controls_on;
 
-  guint n_items;
+  guint         n_items;
 
-  guint item_height;
+  guint         item_height;
 
-  gdouble upper_hack;
+  GtkMenuItem  *selected_item;
 
-  GtkMenuItem *selected_item;
+  gboolean      resize_parent;
+
+  guint         toggle_size;
 };
 
 static GObject *hildon_desktop_popup_menu_constructor (GType gtype,
@@ -87,9 +91,11 @@ static void hildon_desktop_popup_menu_set_property (GObject *object,
                                                     const GValue *value,
                                                     GParamSpec *pspec);
 
+static void hildon_desktop_popup_menu_size_request (GtkWidget *widget, GtkRequisition *req);
+static void hildon_desktop_popup_menu_size_allocate (GtkWidget *widget, GtkAllocation *allocation);
 static gboolean hildon_desktop_popup_menu_motion_notify (GtkWidget *widget, GdkEventMotion *event);
 static gboolean hildon_desktop_popup_menu_release_event (GtkWidget *widget, GdkEventButton *event);
-
+static gboolean hildon_desktop_popup_menu_key_press_event (GtkWidget *widget, GdkEventKey *event);
 static void hildon_desktop_popup_menu_scroll_cb (GtkWidget *widget, HildonDesktopPopupMenu *menu);
 
 static void 
@@ -100,10 +106,14 @@ hildon_desktop_popup_menu_init (HildonDesktopPopupMenu *menu)
   menu->priv->item_height  =
   menu->priv->n_items = 0;
 
+  menu->priv->resize_parent = TRUE;
+
   menu->priv->controls_on = FALSE;
 
   menu->priv->selected_item = NULL;
-}	
+
+  menu->priv->toggle_size = 0;
+}
 
 static void 
 hildon_desktop_popup_menu_class_init (HildonDesktopPopupMenuClass *menu_class)
@@ -113,13 +123,17 @@ hildon_desktop_popup_menu_class_init (HildonDesktopPopupMenuClass *menu_class)
 
   object_class->constructor = hildon_desktop_popup_menu_constructor;
   object_class->finalize    = hildon_desktop_popup_menu_finalize;
-  
+
   object_class->get_property = hildon_desktop_popup_menu_get_property;
   object_class->set_property = hildon_desktop_popup_menu_set_property;
 
   widget_class->motion_notify_event  = hildon_desktop_popup_menu_motion_notify;
   widget_class->button_release_event = hildon_desktop_popup_menu_release_event;
+  widget_class->key_press_event      = hildon_desktop_popup_menu_key_press_event;
 
+  widget_class->size_request = hildon_desktop_popup_menu_size_request;
+  widget_class->size_allocate = hildon_desktop_popup_menu_size_allocate;
+  
   g_type_class_add_private (object_class, sizeof (HildonDesktopPopupMenuPrivate));
 
   signals[SIGNAL_POPUP_RESIZE] =
@@ -130,6 +144,15 @@ hildon_desktop_popup_menu_class_init (HildonDesktopPopupMenuClass *menu_class)
                       NULL, NULL,
                       g_cclosure_marshal_VOID__VOID,
                       G_TYPE_NONE, 0);
+ 
+  signals[SIGNAL_POPUP_SHOW_CONTROLS] =
+        g_signal_new ("show-controls",
+                      G_OBJECT_CLASS_TYPE (object_class),
+                      G_SIGNAL_RUN_LAST,
+                      0,
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__BOOLEAN,
+                      G_TYPE_NONE, 1, G_TYPE_BOOLEAN); 
   
   g_object_class_install_property (object_class,
                                    PROP_POPUP_ITEM_HEIGHT,
@@ -141,6 +164,16 @@ hildon_desktop_popup_menu_class_init (HildonDesktopPopupMenuClass *menu_class)
                                             G_MAXINT,
                                             40,
                                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+  g_object_class_install_property (object_class,
+                                   PROP_POPUP_RESIZE_PARENT,
+                                   g_param_spec_boolean(
+                                           "resize-parent",
+                                           "resize-parent",
+                                           "Whether resize or not parent window of menu",
+                                            TRUE,
+                                            G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
 }
 
 static GObject *
@@ -161,8 +194,8 @@ hildon_desktop_popup_menu_constructor (GType gtype,
   gtk_box_pack_start (GTK_BOX (menu),
                       menu->priv->viewport,
                       FALSE, FALSE, 0);
-  gtk_widget_show (menu->priv->viewport);
 
+  gtk_widget_show (menu->priv->viewport);
 
   gtk_widget_push_composite_child ();
 
@@ -174,14 +207,14 @@ hildon_desktop_popup_menu_constructor (GType gtype,
   
   menu->priv->box_buttons = gtk_hbox_new (TRUE,0);
 
-  menu->priv->scroll_down = gtk_button_new_with_label ("down");
-  menu->priv->scroll_up   = gtk_button_new_with_label ("up");
-
+  menu->priv->scroll_down = gtk_button_new ();
+  menu->priv->scroll_up   = gtk_button_new ();
 
   g_signal_connect (menu->priv->scroll_down,
 		    "clicked",
 		    G_CALLBACK (hildon_desktop_popup_menu_scroll_cb),
 		    (gpointer)menu);
+
   g_signal_connect (menu->priv->scroll_up,
                     "clicked",
                     G_CALLBACK (hildon_desktop_popup_menu_scroll_cb),
@@ -199,7 +232,7 @@ hildon_desktop_popup_menu_constructor (GType gtype,
   gtk_widget_show (menu->priv->scroll_up);
   
   g_object_ref (G_OBJECT (menu->priv->box_buttons));
-  gtk_object_sink (GTK_OBJECT (menu->priv->box_buttons));
+  g_object_ref_sink (GTK_OBJECT (menu->priv->box_buttons));
 		      
   gtk_widget_pop_composite_child ();
 
@@ -213,6 +246,96 @@ hildon_desktop_popup_menu_finalize (GObject *object)
 	
   G_OBJECT_CLASS (hildon_desktop_popup_menu_parent_class)->finalize (object);	
 }	
+static void
+hildon_desktop_popup_menu_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
+{
+  HildonDesktopPopupMenu *menu;
+  HildonDesktopPopupMenuPrivate *priv;
+  GtkRequisition child_requisition;
+  GtkAllocation child_allocation;
+  GList *children, *iter;
+  gint height = 0;
+  
+  menu = HILDON_DESKTOP_POPUP_MENU (widget);
+  priv = menu->priv;
+
+  iter = children = gtk_container_get_children (GTK_CONTAINER (priv->box_items));
+  
+  while (iter)
+  {
+    GtkWidget *child;
+
+    child = iter->data;
+    iter  = iter->next;
+
+    if (GTK_WIDGET_VISIBLE (child))
+    {
+      gtk_widget_size_request (GTK_WIDGET (child), &child_requisition);
+
+      child_allocation.width = allocation->width;
+      child_allocation.height = child_requisition.height;
+      child_allocation.x = 0;
+      child_allocation.y = height;
+	    
+      gtk_menu_item_toggle_size_allocate (GTK_MENU_ITEM (child),
+                                          priv->toggle_size);
+
+      gtk_widget_size_allocate (child, &child_allocation);
+
+      gtk_widget_queue_draw (GTK_WIDGET (child));
+
+      height += child_requisition.height;
+    }
+  }
+
+  g_list_free (children);
+
+  GTK_WIDGET_CLASS (hildon_desktop_popup_menu_parent_class)->size_allocate (widget, allocation);
+}
+
+static void
+hildon_desktop_popup_menu_size_request (GtkWidget *widget, GtkRequisition *req)
+{
+  HildonDesktopPopupMenu *menu;
+  HildonDesktopPopupMenuPrivate *priv;
+  GtkRequisition child_requisition;
+  guint max_toggle_size;
+  GList *children, *iter;
+  
+  g_return_if_fail (HILDON_DESKTOP_IS_POPUP_MENU (widget));
+  g_return_if_fail (req != NULL);
+
+  menu = HILDON_DESKTOP_POPUP_MENU (widget);
+  priv = menu->priv;
+  
+  max_toggle_size = 0;
+
+  iter = children = gtk_container_get_children (GTK_CONTAINER (priv->box_items));
+
+  while (iter)
+  {
+    GtkMenuItem *child;
+    gint toggle_size;
+
+    child = (GtkMenuItem *) iter->data;
+    iter  = iter->next;
+
+    if (GTK_WIDGET_VISIBLE (child))
+    {
+      gtk_widget_size_request (GTK_WIDGET (child), &child_requisition);
+
+      gtk_menu_item_toggle_size_request (child, &toggle_size);
+
+      max_toggle_size = MAX (max_toggle_size, toggle_size);
+    }
+  }
+
+  priv->toggle_size = max_toggle_size;
+
+  g_list_free (children);
+
+  GTK_WIDGET_CLASS (hildon_desktop_popup_menu_parent_class)->size_request (widget, req);
+}
 
 static void 
 hildon_desktop_popup_menu_get_property (GObject *object,
@@ -228,6 +351,10 @@ hildon_desktop_popup_menu_get_property (GObject *object,
       g_value_set_uint (value, menu->priv->item_height);
       break;
 
+    case PROP_POPUP_RESIZE_PARENT:
+      g_value_set_boolean (value, menu->priv->resize_parent);
+      break;
+
    default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -248,6 +375,10 @@ hildon_desktop_popup_menu_set_property (GObject *object,
       menu->priv->item_height = g_value_get_uint (value);
       break;
 
+   case PROP_POPUP_RESIZE_PARENT:
+      menu->priv->resize_parent = g_value_get_boolean (value);
+      break;
+
    default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -262,6 +393,14 @@ hildon_desktop_popup_menu_motion_notify (GtkWidget      *widget,
   GList *menu_items = NULL, *l;
   gint w,h,x,y;
 
+  gtk_widget_get_pointer (GTK_WIDGET (widget), &x, &y);
+
+  w = widget->allocation.width;
+  h = widget->allocation.height;
+
+  if (!((x >= 0) && (x <= w) && (y >= 0) && (y <= h)))
+    return TRUE;    	 
+
   menu_items = 
     gtk_container_get_children (GTK_CONTAINER (menu->priv->box_items));
 
@@ -276,7 +415,10 @@ hildon_desktop_popup_menu_motion_notify (GtkWidget      *widget,
     h = GTK_WIDGET (l->data)->allocation.height;
 
     if ((x >= 0) && (x <= w) && (y >= 0) && (y <= h))
+    {	    
       gtk_item_select (GTK_ITEM (l->data));
+      menu->priv->selected_item = GTK_MENU_ITEM (l->data);
+    }
   }
 
   g_list_free (menu_items);
@@ -288,20 +430,34 @@ static void
 hildon_desktop_popup_menu_scroll_cb (GtkWidget *widget, HildonDesktopPopupMenu *menu)
 {
   gdouble position;
-  gint delta = menu->priv->item_height;
+  gint delta = menu->priv->item_height + 16; /*FIXME: Check items aside the menu item */
   GtkAdjustment *adj = 
     gtk_viewport_get_vadjustment 
       (GTK_VIEWPORT (menu->priv->viewport));
+  GtkRequisition req;
+  gdouble upper_hack;
 
   if (widget == menu->priv->scroll_up)
     delta *= -1;
   
   position = gtk_adjustment_get_value (adj);
 
-  if ((gint)(position + (gdouble)delta) <= menu->priv->upper_hack)
-    gtk_adjustment_set_value (adj, position + (gdouble)delta); 	 
+  GtkWidget *parent = gtk_widget_get_parent (GTK_WIDGET (menu));
 
-  g_debug ("min: %lf max: %lf current: %lf", adj->lower,adj->upper, adj->value);
+  if (parent)
+  {		  
+    gtk_widget_size_request (parent, &req);
+  	  
+    upper_hack = adj->upper - (req.height - menu->priv->item_height);
+
+    if ((gint)(position + (gdouble)delta) <= upper_hack)
+      gtk_adjustment_set_value (adj, position + (gdouble)delta); 	
+    else
+      gtk_adjustment_set_value (adj, upper_hack);	    
+
+    /* NOTE: Don't remove this
+     * g_debug ("min: %lf max: %lf current: %lf upper_hack: %lf", adj->lower,adj->upper, adj->value, upper_hack);*/
+  }
 }	
 
 static gboolean 
@@ -339,6 +495,110 @@ hildon_desktop_popup_menu_release_event (GtkWidget      *widget,
   return TRUE;
 }
 
+static void 
+hildon_desktop_menu_check_scroll_item (HildonDesktopPopupMenu *menu,
+				       GtkScrollType type)
+{
+  GtkAdjustment *adj;	
+  gint visible_y;
+  GtkWidget *view = GTK_WIDGET (menu->priv->selected_item);
+  gdouble upper_hack; 
+  GtkRequisition req;
+
+  GtkWidget *parent = gtk_widget_get_parent (GTK_WIDGET (menu));
+
+  gtk_widget_size_request (parent, &req);
+
+  adj = gtk_viewport_get_vadjustment (GTK_VIEWPORT (menu->priv->viewport));
+  
+  if (parent && GTK_IS_WINDOW (parent)) 
+    upper_hack = adj->upper - (req.height - menu->priv->item_height);
+  else
+    upper_hack = adj->upper;	  
+
+  visible_y = view->allocation.y + menu->priv->item_height;
+ 
+  if (visible_y < (req.height - menu->priv->item_height)) 
+    hildon_desktop_popup_menu_scroll_cb (menu->priv->scroll_up, menu);
+  else
+    hildon_desktop_popup_menu_scroll_cb (menu->priv->scroll_down, menu);
+}
+
+static gboolean 
+hildon_desktop_popup_menu_key_press_event (GtkWidget   *widget,
+					   GdkEventKey *event)
+{
+  HildonDesktopPopupMenu *menu = HILDON_DESKTOP_POPUP_MENU (widget);
+  GList *menu_items = NULL, *l;
+
+  menu_items =
+    gtk_container_get_children (GTK_CONTAINER (menu->priv->box_items)); 
+
+  for (l = menu_items; l != NULL; l = g_list_next (l))
+  {
+    if (l->data == menu->priv->selected_item)
+      break;
+  }
+
+  if (l == NULL)
+    return FALSE;   
+
+  if (event->keyval == GDK_Up ||
+      event->keyval == GDK_KP_Up)
+  {
+    GList *item = l->prev;
+	  
+    while (item)
+    {
+      if (GTK_IS_MENU_ITEM (item->data) && !GTK_IS_SEPARATOR_MENU_ITEM (item->data))
+      {
+        gtk_item_deselect (GTK_ITEM (l->data));
+        gtk_item_select   (GTK_ITEM (item->data));
+	menu->priv->selected_item = GTK_MENU_ITEM (item->data);
+
+	hildon_desktop_menu_check_scroll_item (menu, GTK_SCROLL_STEP_UP);
+	
+	break;
+      }
+
+      item = g_list_previous (item);
+    } 
+    return TRUE;
+  }
+  else
+  if (event->keyval == GDK_Down ||
+      event->keyval == GDK_KP_Down)
+  {
+    GList *item = l->next;
+    
+    while  (item)    
+    {	   
+      if (GTK_IS_MENU_ITEM (item->data) && !GTK_IS_SEPARATOR_MENU_ITEM (item->data))
+      { 
+        gtk_item_deselect (GTK_ITEM (l->data));
+        gtk_item_select   (GTK_ITEM (item->data));
+	menu->priv->selected_item = GTK_MENU_ITEM (item->data);
+
+	hildon_desktop_menu_check_scroll_item (menu, GTK_SCROLL_STEP_DOWN);
+	
+	break;
+      }
+
+      item = g_list_next (item);
+    }
+    return TRUE;
+  }	  
+  else
+  if (event->keyval == GDK_KP_Enter ||
+      event->keyval == GDK_ISO_Enter)
+  {	  
+    if (menu->priv->selected_item)
+      gtk_menu_item_activate (menu->priv->selected_item);
+  }
+
+  return FALSE;	
+}
+
 static void 
 hildon_desktop_popup_menu_show_controls (HildonDesktopPopupMenu *menu)
 {	
@@ -350,6 +610,8 @@ hildon_desktop_popup_menu_show_controls (HildonDesktopPopupMenu *menu)
     gtk_widget_show (menu->priv->box_buttons);
     
     menu->priv->controls_on = TRUE;
+
+    g_signal_emit_by_name (menu, "show-controls", TRUE);
   }
 }
 
@@ -361,6 +623,8 @@ hildon_desktop_popup_menu_hide_controls (HildonDesktopPopupMenu *menu)
     gtk_container_remove (GTK_CONTAINER (menu),
                           menu->priv->box_buttons);	    
     menu->priv->controls_on = FALSE;
+
+    g_signal_emit_by_name (menu, "show-controls", FALSE);
   }
 }
 
@@ -371,21 +635,17 @@ hildon_desktop_popup_menu_parent_size (HildonDesktopPopupMenu *menu)
   gint d_height = 0;
   gboolean show_scroll_controls = FALSE;
   GtkRequisition req;
-  gint separators = 0;
   gint screen_height = gdk_screen_height ();
 
   GtkWidget *parent = gtk_widget_get_parent (GTK_WIDGET (menu));
 
   children = gtk_container_get_children (GTK_CONTAINER (menu->priv->box_items));
 
-  for (l=children; l != NULL; l = g_list_next (l))
+  for (l = children; l != NULL; l = g_list_next (l))
   {
-     if (GTK_IS_SEPARATOR_MENU_ITEM (l->data))
-       separators++;	     
-	  
-     gtk_widget_size_request (GTK_WIDGET (l->data), &req);
+    gtk_widget_size_request (GTK_WIDGET (l->data), &req);
 
-     d_height += req.height;
+    d_height += req.height;
 
     if (d_height > screen_height)
     {	  
@@ -393,32 +653,31 @@ hildon_desktop_popup_menu_parent_size (HildonDesktopPopupMenu *menu)
       show_scroll_controls = TRUE;
       break;
     }
-  }	  
- 
+  }
+
   if (show_scroll_controls)
-  {	  
+  {
     hildon_desktop_popup_menu_show_controls (menu);	  
     gtk_widget_set_size_request 
-       (menu->priv->viewport, req.width, d_height - menu->priv->item_height);
-
-    if (separators == 0)
-      separators++;	    
-    
-    menu->priv->upper_hack = d_height - menu->priv->item_height*separators;
+       (menu->priv->viewport, -1, screen_height - menu->priv->item_height); /*d_height - menu->priv->item_height);*/
   }
   else
-  {	  
+  {
     hildon_desktop_popup_menu_hide_controls (menu);
     gtk_widget_set_size_request
-      (menu->priv->viewport, req.width, d_height);
+      (menu->priv->viewport, -1, d_height);
   }
 
   gtk_widget_queue_resize (menu->priv->viewport);
-  
-  if (GTK_IS_WINDOW (parent))
+ 
+  if (menu->priv->resize_parent && GTK_IS_WINDOW (parent))
   {	  
     gtk_widget_size_request (parent, &req);
-    gtk_widget_set_size_request (parent, req.width, d_height);
+    gtk_widget_set_size_request (parent,
+		    		 req.width,
+				 show_scroll_controls ? 
+				 screen_height : 
+				 d_height);
 
     if (GTK_WIDGET_MAPPED (parent))
     {	    
@@ -429,7 +688,11 @@ hildon_desktop_popup_menu_parent_size (HildonDesktopPopupMenu *menu)
     }
 
     if (GTK_WIDGET_REALIZED (parent))
-      gdk_window_resize (parent->window, req.width, d_height); 
+      gdk_window_resize (parent->window,
+		         req.width,
+			 show_scroll_controls ? 
+			 screen_height : 
+			 d_height);
   }
 
   g_list_free (children);
@@ -439,34 +702,41 @@ void
 hildon_desktop_popup_menu_add_item (HildonDesktopPopupMenu *menu, GtkMenuItem *item)
 {
   GtkRequisition req;
-	
-  g_assert (HILDON_DESKTOP_IS_POPUP_MENU (menu));
+  gint item_width = -1;
+  
+  g_return_if_fail (HILDON_DESKTOP_IS_POPUP_MENU (menu));
   g_return_if_fail (GTK_IS_MENU_ITEM (item));
 
   GtkWidget *parent = gtk_widget_get_parent (GTK_WIDGET (menu));
-	
+
   if (GTK_IS_WINDOW (parent))
   {
-    gtk_widget_size_request (parent, &req);
-
-    if (GTK_IS_SEPARATOR_MENU_ITEM (item))
-    {
-      GtkRequisition req_sep;
-
-      gtk_widget_size_request (GTK_WIDGET (item), &req_sep);
-      gtk_widget_set_size_request (GTK_WIDGET (item), req.width, req_sep.height);	    
-    }
-    else
-      gtk_widget_set_size_request (GTK_WIDGET (item), req.width, menu->priv->item_height);
-
-    gtk_widget_set_size_request (menu->priv->box_buttons, req.width, menu->priv->item_height);
-  }	  
+      gtk_widget_size_request (parent, &req);
+      item_width = req.width;
+  }
+  
+  if (GTK_IS_SEPARATOR_MENU_ITEM (item))
+  {
+    GtkRequisition req_sep;
 
+    gtk_widget_size_request (GTK_WIDGET (item), &req_sep);
+    gtk_widget_set_size_request (GTK_WIDGET (item), item_width, req_sep.height);
+  }
+  else
+  {
+    gtk_widget_set_size_request (GTK_WIDGET (item), item_width, menu->priv->item_height);
+  }
+  
   gtk_box_pack_end (GTK_BOX (menu->priv->box_items),
 		    GTK_WIDGET (item),
 		    FALSE, FALSE, 0);
+
   gtk_widget_show (GTK_WIDGET (item));
 
+  gtk_widget_set_size_request (menu->priv->box_buttons, 
+		  	       item_width, 
+			       menu->priv->item_height);
+
   menu->priv->n_items++;
 
   hildon_desktop_popup_menu_parent_size (menu);
@@ -475,11 +745,10 @@ hildon_desktop_popup_menu_add_item (HildonDesktopPopupMenu *menu, GtkMenuItem *i
 void 
 hildon_desktop_popup_menu_remove_item (HildonDesktopPopupMenu *menu, GtkMenuItem *item)
 {
-	
   g_assert (HILDON_DESKTOP_IS_POPUP_MENU (menu));
   g_return_if_fail (GTK_IS_MENU_ITEM (item));
 
-  gtk_container_remove (GTK_CONTAINER (menu->priv->box_items), GTK_WIDGET (item));	    
+  gtk_container_remove (GTK_CONTAINER (menu->priv->box_items), GTK_WIDGET (item));
  
   menu->priv->n_items--;
 
@@ -489,7 +758,7 @@ hildon_desktop_popup_menu_remove_item (HildonDesktopPopupMenu *menu, GtkMenuItem
 GList *
 hildon_desktop_popup_menu_get_children (HildonDesktopPopupMenu *menu)
 {
-  g_assert (HILDON_DESKTOP_IS_POPUP_MENU (menu));
+  g_return_val_if_fail (HILDON_DESKTOP_IS_POPUP_MENU (menu), NULL);
 	
   GList *list = gtk_container_get_children (GTK_CONTAINER (menu->priv->box_items));
 	
@@ -501,30 +770,53 @@ hildon_desktop_popup_menu_select_item (HildonDesktopPopupMenu *menu, GtkMenuItem
 {
   GList *children = NULL, *l;
 
-  g_assert (HILDON_DESKTOP_IS_POPUP_MENU (menu));
-  g_return_if_fail (GTK_IS_MENU_ITEM (item));
+  g_return_if_fail (HILDON_DESKTOP_IS_POPUP_MENU (menu));
 
   children = gtk_container_get_children (GTK_CONTAINER (menu->priv->box_items));
 
   for (l = children; l != NULL; l = g_list_next (l))
   {
-    if (l->data == item)
+    if (l->data && l->data == item)
     {
       gtk_item_select (GTK_ITEM (item));
       menu->priv->selected_item = item;
-      break;
     }
+    else
+      gtk_item_deselect (GTK_ITEM (l->data));	    
   }
 
   g_list_free (children);
 } 
 
+void
+hildon_desktop_popup_menu_select_first_item (HildonDesktopPopupMenu *menu)
+{
+  GList *children = NULL, *l;
+
+  g_return_if_fail (HILDON_DESKTOP_IS_POPUP_MENU (menu));
+
+  children = gtk_container_get_children (GTK_CONTAINER (menu->priv->box_items));
+
+  for (l = children; l != NULL; l = g_list_next (l))
+  {	  
+    if (!GTK_IS_SEPARATOR_MENU_ITEM (l->data))
+    {
+      if (menu->priv->selected_item != NULL)
+        gtk_item_deselect (GTK_ITEM (menu->priv->selected_item));
+
+      gtk_item_select (GTK_ITEM (l->data));
+      menu->priv->selected_item = GTK_MENU_ITEM (l->data);
+      break;
+    }	    
+  }	  
+}	
+
 void
 hildon_desktop_popup_menu_activate_item (HildonDesktopPopupMenu *menu, GtkMenuItem *item)
 {	
   GList *children = NULL, *l;
 
-  g_assert (HILDON_DESKTOP_IS_POPUP_MENU (menu));
+  g_return_if_fail (HILDON_DESKTOP_IS_POPUP_MENU (menu));
 
   children = gtk_container_get_children (GTK_CONTAINER (menu->priv->box_items));
 
@@ -540,15 +832,21 @@ hildon_desktop_popup_menu_activate_item (HildonDesktopPopupMenu *menu, GtkMenuIt
   g_list_free (children);				  
 }	
 
+const GtkMenuItem *
+hildon_desktop_popup_menu_get_selected_item (HildonDesktopPopupMenu *menu)
+{
+  return menu->priv->selected_item;
+}
+
 void   
 hildon_desktop_popup_menu_scroll_to_selected (HildonDesktopPopupMenu *menu)
 {
   GList *children = NULL, *l;
   gint position = 0, screen_height = gdk_screen_height (); 	 
  
-  if (HILDON_DESKTOP_IS_POPUP_MENU (menu))
-    return;	  
-  
+  if (!HILDON_DESKTOP_IS_POPUP_MENU (menu))
+    return;
+ 
   if (menu->priv->selected_item)
   {    	  
     children = gtk_container_get_children (GTK_CONTAINER (menu->priv->box_items));
@@ -573,3 +871,16 @@ hildon_desktop_popup_menu_scroll_to_selected (HildonDesktopPopupMenu *menu)
 
   }
 }
+
+const GtkWidget *
+hildon_desktop_popup_menu_get_scroll_button_up (HildonDesktopPopupMenu *menu)
+{
+  return menu->priv->scroll_up;
+}	
+
+const GtkWidget *
+hildon_desktop_popup_menu_get_scroll_button_down (HildonDesktopPopupMenu *menu)
+{
+  return menu->priv->scroll_down;
+}	
+
