@@ -44,6 +44,7 @@ enum
 enum
 {
   SIGNAL_POPUP_SHOW,
+  SIGNAL_POPUP_CLOSE,
   POPUP_N_SIGNALS
 };
 
@@ -121,6 +122,28 @@ hildon_desktop_popup_window_init (HildonDesktopPopupWindow *popup)
   popup->priv->attached_widget = NULL;
 }
 
+static gboolean 
+hildon_desktop_popup_window_key_press_event (GtkWidget *widget, GdkEventKey *event)
+{
+  if (GTK_BIN (widget)->child)
+    return gtk_widget_event (GTK_BIN (widget)->child,(GdkEvent *)event);	
+
+  return
+    GTK_WIDGET_CLASS 
+      (hildon_desktop_popup_window_parent_class)->key_press_event (widget, event);	  
+}  
+
+static gboolean 
+hildon_desktop_popup_window_key_press_event_cb (GtkWidget *widget, 
+						GdkEventKey *event,
+						HildonDesktopPopupWindow *popup)
+{
+  if (GTK_BIN (widget)->child)
+    return gtk_widget_event (GTK_BIN (widget)->child,(GdkEvent *)event);
+
+  return FALSE;
+}
+
 static void 
 hildon_desktop_popup_window_class_init (HildonDesktopPopupWindowClass *popup_class)
 {
@@ -135,7 +158,7 @@ hildon_desktop_popup_window_class_init (HildonDesktopPopupWindowClass *popup_cla
   widget_class->motion_notify_event     = hildon_desktop_popup_window_motion_notify;
   widget_class->leave_notify_event      = hildon_desktop_popup_window_leave_notify;
   widget_class->button_release_event    = hildon_desktop_popup_window_button_release_event;
-
+  widget_class->key_press_event 	= hildon_desktop_popup_window_key_press_event;
 	  
   widget_class->realize    = hildon_desktop_popup_window_realize;
   widget_class->unrealize  = hildon_desktop_popup_window_unrealize;
@@ -154,6 +177,15 @@ hildon_desktop_popup_window_class_init (HildonDesktopPopupWindowClass *popup_cla
                       NULL, NULL,
                       g_cclosure_marshal_VOID__VOID,
                       G_TYPE_NONE, 0);
+ 
+  signals[SIGNAL_POPUP_SHOW] =
+        g_signal_new ("popdown-window",
+                      G_OBJECT_CLASS_TYPE (object_class),
+                      G_SIGNAL_RUN_LAST,
+                      G_STRUCT_OFFSET (HildonDesktopPopupWindowClass,popdown_window),
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__VOID,
+                      G_TYPE_NONE, 0); 
 
   g_object_class_install_property (object_class,
                                    PROP_POPUP_N_PANES,
@@ -237,6 +269,11 @@ hildon_desktop_popup_window_constructor (GType gtype,
 		      G_CALLBACK (hildon_desktop_popup_window_composited_leave_notify),
 		      (gpointer)popup);
 
+    g_signal_connect (popup->priv->extra_panes[i],
+		      "key-press-event",
+		      G_CALLBACK (hildon_desktop_popup_window_key_press_event_cb),
+		      (gpointer)popup);
+
     /*FIXME: NO FOCUS FOR ANY WINDOW!!!!!!!!!!!!!!!!!! */
   }
 
@@ -864,18 +901,18 @@ void
 hildon_desktop_popup_window_jump_to_pane (HildonDesktopPopupWindow *popup, gint pane)
 {
   g_assert (HILDON_DESKTOP_IS_POPUP_WINDOW (popup));
-
-  if (pane >= popup->priv->n_extra_panes)
+  
+  if (pane >= (gint) popup->priv->n_extra_panes)
     pane = popup->priv->n_extra_panes -1;	  
-	
+  
   if (GTK_WIDGET_VISIBLE (GTK_WIDGET (popup)))
   {
     if (pane <= -1)
-    {
+    { 
       if (popup->priv->have_xgrab)	    
         return;
       else
-      {
+      { 
         gtk_grab_remove (popup->priv->pane_with_grab);	      
 
 	popup_grab_on_window (GTK_WIDGET (popup)->window, GDK_CURRENT_TIME, TRUE);
@@ -887,7 +924,7 @@ hildon_desktop_popup_window_jump_to_pane (HildonDesktopPopupWindow *popup, gint
       }
     }
     else
-    {
+    { 
       GtkWidget *widget_with_grab;
 	    
       if (popup->priv->have_xgrab)
@@ -967,6 +1004,8 @@ hildon_desktop_popup_window_popdown (HildonDesktopPopupWindow *popup)
 
   for (i=0; i < popup->priv->n_extra_panes; i++)
     gtk_grab_remove (popup->priv->extra_panes[i]);	  
+
+  g_signal_emit_by_name (popup, "popdown-window");
 }
 
 void 
