@@ -128,7 +128,8 @@ enum
 enum
 {
   PROP_0,
-  PROP_INIT_DBUS
+  PROP_INIT_DBUS,
+  PROP_DESKTOP_WINDOW
 };
 
 static gint hdwm_signals[HDWM_SIGNALS];
@@ -219,6 +220,8 @@ struct _HDWMPrivate   /* Our main struct */
 
   GtkWidget              *all_menu;
 
+  Window                  desktop_window;
+
   /* stack for the launch banner messages. 
    * Needed to work round gtk(hindon)_infoprint issues.
    */
@@ -763,14 +766,14 @@ mce_handler (DBusConnection *conn,
   if (!member || member[0] == '\0')
     return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 
-  if (strcmp (HOME_LONG_PRESS, member) == 0 && !hd_wm_modal_windows_present())
+  if (g_str_equal (HOME_LONG_PRESS, member) && !hd_wm_modal_windows_present())
   {
     g_signal_emit_by_name (hdwm, "long-key-press");
 
     return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
   }
   
-  if (strcmp (HOME_PRESS, member) == 0 && !hd_wm_modal_windows_present())
+  if (g_str_equal (HOME_PRESS, member) && !hd_wm_modal_windows_present())
   {
     hd_wm_activate (HD_TN_ACTIVATE_MAIN_MENU);
 
@@ -973,7 +976,7 @@ hd_wm_class_init (HDWMClass *hdwm_class)
              	     G_TYPE_POINTER);
 
   hdwm_signals[HDWM_SHOW_A_MENU_SIGNAL] = 
-	g_signal_new("show_menu",
+	g_signal_new("show-menu",
 		     G_OBJECT_CLASS_TYPE(object_class),
 		     G_SIGNAL_RUN_LAST,
 		     G_STRUCT_OFFSET (HDWMClass,show_menu),
@@ -1029,19 +1032,33 @@ hd_wm_class_init (HDWMClass *hdwm_class)
                                                         "Max width when horizontal",
 	                                                TRUE,
 							G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+  g_object_class_install_property (object_class,
+                                   PROP_DESKTOP_WINDOW,
+                                   g_param_spec_int("desktop-window",
+                                                    "desktop window",
+                                                    "Current desktop window",
+	                                            0,
+                                                    G_MAXINT,
+                                                    0,
+						    G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 }
 
 static void 
 hd_wm_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
 {
-  HDWM *hdwm = HD_WM (object);	
-	
+  HDWM *hdwm = HD_WM (object);
+
   switch (prop_id)
   {
     case PROP_INIT_DBUS:
       g_value_set_boolean (value, hdwm->priv->init_dbus);
       break;
-      
+
+    case PROP_DESKTOP_WINDOW:
+      g_value_set_int (value, (int)hdwm->priv->desktop_window);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1059,6 +1076,10 @@ hd_wm_set_property (GObject *object, guint prop_id, const GValue *value, GParamS
       hdwm->priv->init_dbus = g_value_get_boolean (value);
       break;      
 	  
+    case PROP_DESKTOP_WINDOW:
+      hdwm->priv->desktop_window = (Window)g_value_get_int (value);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1103,6 +1124,8 @@ hd_wm_init (HDWM *hdwm)
 	
   memset(hdwm->priv, 0, sizeof(HDWMPrivate));
 
+  hdwm->priv->init_dbus = TRUE;
+  
   /* Check for configurable lowmem values. */
 
   hdwm->priv->lowmem_min_distance
@@ -1236,6 +1259,12 @@ hd_wm_top_item (HDEntryInfo *info)
     gdk_error_trap_pop();
 
   }
+
+  if (info->type == HD_ENTRY_DESKTOP)
+  {
+    hd_wm_top_desktop ();	  
+    g_signal_emit_by_name (hdwm, "entry_info_stack_changed", info);
+  }	  
   else
     g_debug  ("### Invalid window type ###\n");
 }
@@ -1762,13 +1791,10 @@ hd_wm_process_mb_current_app_window (HDWM *hdwm)
      * deactivate any active buttons.
      */
       
-    HDEntryInfo * info = NULL;
-      /*  = hn_app_switcher_get_home_entry_info (hdwm->priv->app_switcher);*/
+    HDEntryInfo * info = hdwm->priv->home_info;
       
     if (info)
       g_signal_emit_by_name (hdwm,"entry_info_stack_changed",info);
-    else
-      g_debug  ("***************** No Home info yet");
   }
   
 out:
@@ -2274,7 +2300,7 @@ hd_wm_activate_window (guint32 what, GdkWindow *window)
       return;
     case HD_TN_ACTIVATE_MAIN_MENU:
       g_debug  ("activating main menu: signal");
-      g_signal_emit_by_name (hdwm, "show_menu");
+      g_signal_emit_by_name (hdwm, "show-menu");
       return;
     case HD_TN_ACTIVATE_LAST_APP_WINDOW:
       hdwm->priv->has_focus = FALSE;
