@@ -30,12 +30,19 @@
 #include <hildon/hildon-help.h>
 #endif
 
+#ifdef HAVE_X_COMPOSITE
+#include <X11/extensions/Xrender.h>
+#include <X11/extensions/Xcomposite.h>
+#endif
+
 #ifdef HAVE_LIBOSSO
 #include <libosso.h>
 #endif
 
+
+#include <libhildonwm/hd-wm.h>
+
 #include <libhildondesktop/hildon-home-area.h>
-#include <libhildondesktop/hildon-home-titlebar.h>
 #include <libhildondesktop/hildon-desktop-home-item.h>
 
 #include <gtk/gtkcheckmenuitem.h>
@@ -74,6 +81,9 @@
 #define HD_HOME_WINDOW_STYLE_NORTH_BORDER   "HildonHomeTitleBar"
 #define HD_HOME_WINDOW_STYLE_WEST_BORDER    "HildonHomeLeftEdge"
 
+#define HH_AREA_CONFIGURATION_FILE        ".osso/hildon-desktop/home-layout.conf"
+#define HH_AREA_GLOBAL_CONFIGURATION_FILE "/etc/hildon-desktop/home-layout.conf"
+
 #include "hd-home-window.h"
 
 #define HD_HOME_WINDOW_GET_PRIVATE(obj) \
@@ -89,6 +99,9 @@ struct _HDHomeWindowPrivate
 
   gboolean          layout_mode_sucks;
 
+  GtkWidget        *normal_menu;
+  GtkWidget        *layout_menu;
+
   GtkWidget        *settings_item;
   GtkWidget        *settings_menu;
   GtkWidget        *layout_mode_item;
@@ -101,6 +114,11 @@ struct _HDHomeWindowPrivate
 
   HDHomeBackground *background;
   HDHomeBackground *previous_background;
+
+  gboolean          is_inactive;
+
+  gboolean          selecting_applets;
+  guint             save_area_timeout;
 };
 
 /* Properties */
@@ -161,6 +179,9 @@ static void
 hd_home_window_layout_insensitive_press_cb (HDHomeWindow *window);
 #endif
 
+static void
+hd_home_window_layout_mode_activate (HDHomeWindow *window);
+
 static GtkWidget *
 hd_home_window_build_main_menu (HDHomeWindow *window);
 
@@ -171,10 +192,16 @@ static void
 hd_home_window_ensure_menu_status (HDHomeWindow *window);
 
 static void
-hd_home_window_layout_mode_accept (HildonHomeWindow *window);
+hd_home_window_cancel_layout (HDHomeWindow *window);
 
 static void
-hd_home_window_layout_mode_cancel (HildonHomeWindow *window);
+hd_home_window_accept_layout (HDHomeWindow *window);
+
+static void
+hd_home_window_layout_mode_accept (HDHomeWindow *window);
+
+static void
+hd_home_window_layout_mode_cancel (HDHomeWindow *window);
 
 static void
 hd_home_window_snap_toggled (HDHomeWindow *window, GtkCheckMenuItem *item);
@@ -193,6 +220,12 @@ static gboolean
 hd_home_window_map_event (GtkWidget    *widget,
                           GdkEventAny  *event);
 
+static void
+hd_home_window_realize (GtkWidget *widget);
+
+static void
+hd_home_window_unrealize (GtkWidget *widget);
+
 static gboolean
 hd_home_window_key_press_event (GtkWidget *widget,
                                 GdkEventKey *event);
@@ -211,6 +244,41 @@ hd_home_window_show_information_banner (HDHomeWindow *window,
                                         const gchar *text);
 #endif
 
+static void
+hd_home_window_background (HDHomeWindow        *window,
+                           gboolean             background);
+
+static void
+hd_home_window_system_inactivity (HDHomeWindow     *window,
+                                  gboolean              is_inactive);
+
+static void
+hd_home_window_area_add (HDHomeWindow *window);
+
+static void
+hd_home_window_area_remove (HDHomeWindow *window);
+
+static void
+hd_home_window_area_changed (HDHomeWindow *window);
+
+static void
+hd_home_window_layout_mode_started (HDHomeWindow *window);
+
+static void
+hd_home_window_layout_mode_ended (HDHomeWindow *window);
+
+static void
+hd_home_window_select_applets (HDHomeWindow *window);
+
+static void
+hd_home_window_load_area_layout (HDHomeWindow *window);
+
+static void
+hd_home_window_button1_clicked (HildonHomeWindow *window);
+
+static void
+hd_home_window_button2_clicked (HildonHomeWindow *window);
+
 #if 0
 static void
 hd_home_window_adjust_alpha_activate (HDHomeWindow *window);
@@ -236,9 +304,16 @@ hd_home_window_class_init (HDHomeWindowClass *window_class)
   widget_class->map_event = hd_home_window_map_event;
   widget_class->style_set = hd_home_window_style_set;
   widget_class->key_press_event = hd_home_window_key_press_event;
+  widget_class->realize   = hd_home_window_realize;
+  widget_class->unrealize = hd_home_window_unrealize;
 
-  hhwindow_class->layout_mode_accept = hd_home_window_layout_mode_accept;
-  hhwindow_class->layout_mode_cancel = hd_home_window_layout_mode_cancel;
+  hhwindow_class->button1_clicked = hd_home_window_button1_clicked;
+  hhwindow_class->button2_clicked = hd_home_window_button2_clicked;
+
+  window_class->layout_mode_accept = hd_home_window_layout_mode_accept;
+  window_class->layout_mode_cancel = hd_home_window_layout_mode_cancel;
+  window_class->background         = hd_home_window_background;
+  window_class->system_inactivity  = hd_home_window_system_inactivity;
 
   pspec = g_param_spec_pointer ("osso-context",
                                 "Osso Context",
@@ -256,7 +331,7 @@ hd_home_window_class_init (HDHomeWindowClass *window_class)
   g_object_class_install_property (object_class,
                                    PROP_BACKGROUND,
                                    pspec);
-  
+
   pspec = g_param_spec_boolean ("layout-mode-sucks",
                                 "Layout mode sucks",
                                 "Whether or not the layout mode sucks",
@@ -266,6 +341,73 @@ hd_home_window_class_init (HDHomeWindowClass *window_class)
                                    PROP_LAYOUT_MODE_SUCKS,
                                    pspec);
 
+  g_signal_new ("background",
+                G_OBJECT_CLASS_TYPE (object_class),
+                G_SIGNAL_RUN_FIRST,
+                G_STRUCT_OFFSET (HDHomeWindowClass, background),
+                NULL,
+                NULL,
+                g_cclosure_marshal_VOID__BOOLEAN,
+                G_TYPE_NONE,
+                1,
+                G_TYPE_BOOLEAN);
+
+  g_signal_new ("system-inactivity",
+                G_OBJECT_CLASS_TYPE (object_class),
+                G_SIGNAL_RUN_FIRST,
+                G_STRUCT_OFFSET (HDHomeWindowClass, system_inactivity),
+                NULL,
+                NULL,
+                g_cclosure_marshal_VOID__BOOLEAN,
+                G_TYPE_NONE,
+                1,
+                G_TYPE_BOOLEAN);
+
+  g_signal_new ("lowmem",
+                G_OBJECT_CLASS_TYPE (object_class),
+                G_SIGNAL_RUN_FIRST,
+                G_STRUCT_OFFSET (HDHomeWindowClass, lowmem),
+                NULL,
+                NULL,
+                g_cclosure_marshal_VOID__BOOLEAN,
+                G_TYPE_NONE,
+                1,
+                G_TYPE_BOOLEAN);
+
+  g_signal_new ("layout-mode-accept",
+                G_OBJECT_CLASS_TYPE (object_class),
+                G_SIGNAL_RUN_FIRST,
+                G_STRUCT_OFFSET (HDHomeWindowClass, layout_mode_accept),
+                NULL,
+                NULL,
+                g_cclosure_marshal_VOID__VOID,
+                G_TYPE_NONE,
+                0);
+
+  g_signal_new ("layout-mode-cancel",
+                G_OBJECT_CLASS_TYPE (object_class),
+                G_SIGNAL_RUN_FIRST,
+                G_STRUCT_OFFSET (HDHomeWindowClass, layout_mode_cancel),
+                NULL,
+                NULL,
+                g_cclosure_marshal_VOID__VOID,
+                G_TYPE_NONE,
+                0);
+
+  g_signal_new ("io-error",
+                G_OBJECT_CLASS_TYPE (object_class),
+                G_SIGNAL_RUN_FIRST,
+                G_STRUCT_OFFSET (HDHomeWindowClass, io_error),
+                NULL,
+                NULL,
+                g_cclosure_marshal_VOID__POINTER,
+                G_TYPE_NONE,
+                1,
+                G_TYPE_POINTER);
+
+
+
+
   g_type_class_add_private (window_class, sizeof (HDHomeWindowPrivate));
 
 }
@@ -283,53 +425,64 @@ hd_home_window_constructor (GType                   gtype,
 {
   GObject              *retval;
   HDHomeWindow         *window;
+  HDHomeWindowPrivate  *priv;
   HildonHomeWindow     *hhwindow;
-  GtkWidget            *titlebar;
-  GtkWidget            *menu;
   GtkWidget            *area;
   HDHomeBackground     *background = NULL;
   GError               *error = NULL;
   gchar                *conffile;
 
-
   retval = G_OBJECT_CLASS (hd_home_window_parent_class)->constructor (gtype,
                                                                       n_params,
                                                                       params);
 
   window = HD_HOME_WINDOW (retval);
   hhwindow = HILDON_HOME_WINDOW (retval);
+  priv = HD_HOME_WINDOW_GET_PRIVATE (retval);
 
-  titlebar = hildon_home_window_get_titlebar (hhwindow);
+  priv->normal_menu = hd_home_window_build_main_menu (window);
+  hildon_home_window_set_menu (hhwindow, GTK_MENU (priv->normal_menu));
 
-  menu = hd_home_window_build_main_menu (window);
-  hildon_home_titlebar_set_menu (HILDON_HOME_TITLEBAR (titlebar), menu);
-  
-  menu = hd_home_window_build_layout_menu (window);
-  hildon_home_titlebar_set_layout_menu (HILDON_HOME_TITLEBAR (titlebar), menu);
+  priv->layout_menu = hd_home_window_build_layout_menu (window);
 
-  hildon_home_titlebar_set_menu_title (HILDON_HOME_TITLEBAR (titlebar),
-                                       HH_MENU_TITLE);
-  hildon_home_titlebar_set_layout_menu_title (HILDON_HOME_TITLEBAR (titlebar),
-                                              HH_MENU_LAYOUT_TITLE);
+  g_object_set (window,
+                "menu", priv->normal_menu,
+                "title", HH_MENU_TITLE,
+                NULL);
 
-  area = hildon_home_window_get_area (hhwindow);
+  area = hildon_home_area_new ();
+  gtk_widget_show (area);
+
+  gtk_container_add (GTK_CONTAINER (retval), area);
+  HILDON_DESKTOP_WINDOW (retval)->container = GTK_CONTAINER (area);
 
   g_signal_connect_swapped (area, "add",
-                            G_CALLBACK (hd_home_window_ensure_menu_status),
+                            G_CALLBACK (hd_home_window_area_add),
                             window);
-  
+
   g_signal_connect_swapped (area, "remove",
-                            G_CALLBACK (hd_home_window_ensure_menu_status),
+                            G_CALLBACK (hd_home_window_area_remove),
                             window);
 
   g_signal_connect_swapped (area, "layout-mode-start",
                             G_CALLBACK (hd_home_window_show_layout_mode_banner),
                             window);
 
-  g_signal_connect_swapped (area, "layout-mode-ended",
-                            G_CALLBACK (hd_home_window_destroy_banner),
+  g_signal_connect_swapped (area, "layout-mode-started",
+                            G_CALLBACK (hd_home_window_layout_mode_started),
                             window);
-  
+
+  g_signal_connect_swapped (area, "layout-mode-ended",
+                            G_CALLBACK (hd_home_window_layout_mode_ended),
+                            window);
+
+  if (priv->layout_mode_sucks)
+    {
+      g_signal_connect_swapped (area, "applet-change-end",
+                                G_CALLBACK (hd_home_window_area_changed),
+                                window);
+    }
+
   background = g_object_new (HD_TYPE_HOME_BACKGROUND, NULL);
 
   conffile = g_build_path (G_DIR_SEPARATOR_S,
@@ -392,9 +545,10 @@ error:
   if (background)
     hd_home_window_set_background (window, background);
 
+  hd_home_window_load_area_layout (window);
+
   /* Necessary to avoid the default background to be reset */
   gtk_widget_set_app_paintable (GTK_WIDGET (window), TRUE);
-/*  gtk_widget_set_double_buffered (GTK_WIDGET (window), FALSE);*/
 
   return retval;
 }
@@ -450,11 +604,11 @@ hd_home_window_get_property (GObject    *gobject,
       g_value_set_pointer (value, priv->osso_context);
       break;
 #endif
-    
+
     case PROP_LAYOUT_MODE_SUCKS:
       g_value_set_boolean (value, priv->layout_mode_sucks);
       break;
-      
+
     case PROP_BACKGROUND:
       g_value_set_object (value, priv->background);
       break;
@@ -498,7 +652,7 @@ background_apply_callback (HDHomeBackground *background,
 
   gdk_window_clear (GTK_WIDGET (window)->window);
   gtk_widget_queue_draw (GTK_WIDGET (window));
-  
+
   priv = HD_HOME_WINDOW_GET_PRIVATE (window);
 
   if (background != priv->background)
@@ -507,7 +661,7 @@ background_apply_callback (HDHomeBackground *background,
         g_object_unref (priv->background);
       priv->background = g_object_ref (background);
     }
- 
+
 }
 
 static void
@@ -531,9 +685,9 @@ background_apply_and_save_callback (HDHomeBackground *background,
                            HD_DESKTOP_USER_PATH,
                            HD_HOME_BACKGROUND_CONF_FILE,
                            NULL);
-  
+
   priv = HD_HOME_WINDOW_GET_PRIVATE (window);
-  
+
   hd_home_background_save (priv->background,
                            conffile,
                            &save_error);
@@ -550,6 +704,44 @@ background_apply_and_save_callback (HDHomeBackground *background,
 
 }
 
+static void
+hd_home_window_realize (GtkWidget *widget)
+{
+  HDWM *wm;
+
+  GTK_WIDGET_CLASS (hd_home_window_parent_class)->realize (widget);
+
+#ifdef HAVE_X_COMPOSITE
+  XCompositeRedirectWindow (GDK_DISPLAY (),
+                            GDK_WINDOW_XID (widget->window),
+                            CompositeRedirectAutomatic);
+#endif
+
+  wm = hd_wm_get_singleton ();
+  g_object_set (wm,
+                "desktop-window", (gint)GDK_WINDOW_XID (widget->window),
+                NULL);
+
+}
+
+static void
+hd_home_window_unrealize (GtkWidget *widget)
+{
+  HDWM *wm;
+#ifdef HAVE_X_COMPOSITE
+  XCompositeUnredirectWindow (GDK_DISPLAY (),
+                              GDK_WINDOW_XID (widget->window),
+                              CompositeRedirectAutomatic);
+#endif
+
+  wm = hd_wm_get_singleton ();
+  g_object_set (wm,
+                "desktop-window", (gint)None,
+                NULL);
+
+  GTK_WIDGET_CLASS (hd_home_window_parent_class)->unrealize (widget);
+}
+
 static gboolean
 hd_home_window_map_event (GtkWidget    *widget,
                           GdkEventAny  *event)
@@ -584,7 +776,7 @@ hd_home_window_get_pixmap_name (HDHomeWindow *window,
                                      key,
                                      NULL,
                                      G_TYPE_NONE);
-  
+
   if (style && style->rc_style->bg_pixmap_name[0])
     return style->rc_style->bg_pixmap_name[0];
 
@@ -608,15 +800,15 @@ hd_home_window_style_set (GtkWidget *widget, GtkStyle *old_style)
                                                                old_style);
   north_border = hd_home_window_get_pixmap_name (window,
                                                  HD_HOME_WINDOW_STYLE_NORTH_BORDER);
-  
+
   west_border  = hd_home_window_get_pixmap_name (window,
                                                  HD_HOME_WINDOW_STYLE_WEST_BORDER);
-  
+
 
   /* avoid resetting the background when the window is exposed for the
    * first time
    */
-  if (!old_style || 
+  if (!old_style ||
       !(north_border && g_str_equal (priv->north_border, north_border)) ||
       !(west_border && g_str_equal (priv->west_border, west_border)))
     {
@@ -666,29 +858,19 @@ static gboolean
 hd_home_window_key_press_event (GtkWidget *widget,
                                 GdkEventKey *event)
 {
-  GtkWidget *titlebar = 
-      hildon_home_window_get_titlebar (HILDON_HOME_WINDOW (widget));
-  GtkWidget *area = 
-      hildon_home_window_get_area (HILDON_HOME_WINDOW (widget));
-
   switch (event->keyval)
     {
       case HILDON_HARDKEY_MENU:
-          hildon_home_titlebar_toggle_menu (HILDON_HOME_TITLEBAR (titlebar));
+          hildon_home_window_toggle_menu (HILDON_HOME_WINDOW (widget));
           break;
       case HILDON_HARDKEY_ESC:
-          /* FIXME: Have a signal in HomeWindow instead */
-          if (hildon_home_area_get_layout_mode (HILDON_HOME_AREA(area)))
-            g_signal_emit_by_name (G_OBJECT (titlebar), "layout-cancel");
+          hd_home_window_cancel_layout (HD_HOME_WINDOW (widget));
 
           break;
       default:
-          if (GTK_WIDGET_CLASS (hd_home_window_parent_class)->key_press_event)
             return GTK_WIDGET_CLASS (
                      hd_home_window_parent_class)->key_press_event (widget,
                                                                     event);
-          else
-            return FALSE;
     }
 
   return TRUE;
@@ -710,7 +892,7 @@ hd_home_window_build_main_menu (HDHomeWindow *window)
   menu_item = gtk_menu_item_new_with_label (HH_MENU_SELECT);
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
   g_signal_connect_swapped (menu_item, "activate",
-                            G_CALLBACK (hildon_home_window_select_applets),
+                            G_CALLBACK (hd_home_window_select_applets),
                             window);
   gtk_widget_show (menu_item);
 /*  priv->select_applets_item = menu_item;*/
@@ -773,7 +955,7 @@ hd_home_window_build_main_menu (HDHomeWindow *window)
       /* layout mode */
       menu_item = gtk_menu_item_new_with_label (HH_MENU_EDIT_LAYOUT);
       g_signal_connect_swapped (menu_item, "activate",
-                                G_CALLBACK (hildon_home_window_layout_mode_activate),
+                                G_CALLBACK (hd_home_window_layout_mode_activate),
                                 window);
 #ifdef HAVE_LIBHILDON
       hildon_helper_set_insensitive_message (menu_item, HH_LAYOUT_UNAVAIL_BANNER);
@@ -809,7 +991,6 @@ hd_home_window_build_main_menu (HDHomeWindow *window)
       gtk_widget_show (menu_item);
     }
 
-  
   menu_item = gtk_menu_item_new_with_label ("Adjust transparency");
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
   g_signal_connect_swapped (menu_item, "activate",
@@ -834,21 +1015,21 @@ hd_home_window_build_layout_menu (HDHomeWindow *window)
   mi = gtk_menu_item_new_with_label (HH_MENU_LAYOUT_SELECT);
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
   g_signal_connect_swapped (mi, "activate",
-                            G_CALLBACK (hildon_home_window_select_applets),
+                            G_CALLBACK (hd_home_window_select_applets),
                             window);
   gtk_widget_show (mi);
 
   mi = gtk_menu_item_new_with_label (HH_MENU_LAYOUT_ACCEPT);
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
   g_signal_connect_swapped (mi, "activate",
-                            G_CALLBACK (hildon_home_window_accept_layout),
+                            G_CALLBACK (hd_home_window_accept_layout),
                             window);
   gtk_widget_show (mi);
 
   mi = gtk_menu_item_new_with_label (HH_MENU_LAYOUT_CANCEL);
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
   g_signal_connect_swapped (mi, "activate",
-                            G_CALLBACK (hildon_home_window_cancel_layout),
+                            G_CALLBACK (hd_home_window_cancel_layout),
                             window);
   gtk_widget_show (mi);
 
@@ -857,8 +1038,8 @@ hd_home_window_build_layout_menu (HDHomeWindow *window)
   g_signal_connect_swapped (mi, "toggled",
                             G_CALLBACK (hd_home_window_snap_toggled),
                             window);
-  area = hildon_home_window_get_area (HILDON_HOME_WINDOW (window));
-  if (area)
+  area = GTK_BIN (window)->child;
+  if (HILDON_IS_HOME_AREA (area))
     {
       gboolean snap_to_grid;
       g_object_get (G_OBJECT (area), "snap-to-grid", &snap_to_grid, NULL);
@@ -874,7 +1055,7 @@ hd_home_window_build_layout_menu (HDHomeWindow *window)
                             window);
   gtk_widget_show (mi);
 #endif
-  
+
   return menu;
 }
 
@@ -976,7 +1157,7 @@ hd_home_window_calibration_activate (HDHomeWindow *window)
 {
   hd_home_window_applet_activate (window,
                                   HCP_PLUGIN_PATH_CALIBRATION);
-      
+
 }
 #endif
 
@@ -1007,20 +1188,184 @@ hd_home_window_layout_insensitive_press_cb (HDHomeWindow *window)
 }
 #endif
 
+static void
+hd_home_window_save_area_layout (HDHomeWindow *window)
+{
+  GtkWidget    *area = GTK_BIN (window)->child;
+
+  if (HILDON_IS_HOME_AREA (area))
+    {
+      GError *error = NULL;
+      gchar *filename = g_build_filename (g_get_home_dir (),
+                                          HH_AREA_CONFIGURATION_FILE,
+                                          NULL);
+
+      hildon_home_area_save_configuration (HILDON_HOME_AREA (area),
+                                           filename,
+                                           &error);
+
+      if (error)
+        {
+          g_signal_emit_by_name (window, "io-error", error);
+          g_error_free (error);
+        }
+
+      g_free (filename);
+    }
+}
+
+static void
+hd_home_window_load_area_layout (HDHomeWindow *window)
+{
+  GtkWidget    *area = GTK_BIN (window)->child;
+
+  if (HILDON_IS_HOME_AREA (area))
+    {
+      GError   *error = NULL;
+      gchar    *user_filename = NULL;
+      gchar    *filename = NULL;
+
+      user_filename = g_build_filename (g_get_home_dir (),
+                                        HH_AREA_CONFIGURATION_FILE,
+                                        NULL);
+
+      if (g_file_test (user_filename, G_FILE_TEST_EXISTS))
+        filename = user_filename;
+      else
+        filename = HH_AREA_GLOBAL_CONFIGURATION_FILE;
+
+      hildon_home_area_load_configuration (HILDON_HOME_AREA (area),
+                                           filename,
+                                           &error);
+
+      if (error)
+        {
+          g_signal_emit_by_name (window, "io-error", error);
+          g_error_free (error);
+        }
+
+      g_free (user_filename);
+    }
+
+}
+
+static void
+hd_home_window_button1_clicked (HildonHomeWindow *window)
+{
+  hd_home_window_accept_layout (HD_HOME_WINDOW (window));
+}
+
+static void
+hd_home_window_button2_clicked (HildonHomeWindow *window)
+{
+  hd_home_window_cancel_layout (HD_HOME_WINDOW (window));
+}
+
+
 static void
 hd_home_window_snap_toggled (HDHomeWindow *window, GtkCheckMenuItem *item)
 {
   GtkWidget *area;
 
-  area = hildon_home_window_get_area (HILDON_HOME_WINDOW (window));
+  area = GTK_BIN (window)->child;
 
-  if (area)
+  if (HILDON_IS_HOME_AREA (area))
     g_object_set (area,
                   "snap-to-grid", gtk_check_menu_item_get_active (item),
                   NULL);
 
 }
 
+static void
+hd_home_window_area_add (HDHomeWindow *window)
+{
+  HDHomeWindowPrivate  *priv;
+  GtkWidget            *area;
+
+  priv = HD_HOME_WINDOW_GET_PRIVATE (window);
+  area = GTK_BIN (window)->child;
+
+  if (!priv->layout_mode_sucks)
+    {
+      if (priv->selecting_applets &&
+          !hildon_home_area_get_layout_mode (HILDON_HOME_AREA (area)))
+        hildon_home_area_set_layout_mode (HILDON_HOME_AREA (area), TRUE);
+
+    }
+
+  hd_home_window_ensure_menu_status (window);
+}
+
+static void
+hd_home_window_area_remove (HDHomeWindow *window)
+{
+  HDHomeWindowPrivate  *priv;
+  GtkWidget            *area;
+
+  priv = HD_HOME_WINDOW_GET_PRIVATE (window);
+  area = GTK_BIN (window)->child;
+
+  if (!hildon_home_area_get_layout_mode (HILDON_HOME_AREA (area)))
+    hd_home_window_save_area_layout (window);
+
+  hd_home_window_ensure_menu_status (window);
+
+}
+
+static void
+hd_home_window_layout_mode_started (HDHomeWindow *window)
+{
+  HDHomeWindowPrivate  *priv = HD_HOME_WINDOW_GET_PRIVATE (window);
+
+  g_object_set (window,
+                "title", HH_MENU_LAYOUT_TITLE,
+                "menu",  priv->layout_menu,
+                NULL);
+}
+
+static void
+hd_home_window_layout_mode_ended (HDHomeWindow *window)
+{
+  HDHomeWindowPrivate  *priv = HD_HOME_WINDOW_GET_PRIVATE (window);
+
+  hd_home_window_destroy_banner (window);
+
+  g_object_set (window,
+                "title", HH_MENU_TITLE,
+                "menu",  priv->normal_menu,
+                NULL);
+
+}
+
+static gboolean
+hd_home_window_save_area_timeout (HDHomeWindow *window)
+{
+  HDHomeWindowPrivate  *priv;
+
+  priv = HD_HOME_WINDOW_GET_PRIVATE (window);
+  priv->save_area_timeout = 0;
+
+  hd_home_window_save_area_layout (window);
+
+  return FALSE;
+}
+
+
+#define SAVE_TIMEOUT            1000
+static void
+hd_home_window_area_changed (HDHomeWindow *window)
+{
+  HDHomeWindowPrivate  *priv;
+
+  priv = HD_HOME_WINDOW_GET_PRIVATE (window);
+  if (!priv->save_area_timeout)
+    priv->save_area_timeout = g_timeout_add (SAVE_TIMEOUT,
+                                             (GSourceFunc)
+                                             hd_home_window_save_area_timeout,
+                                             window);
+}
+#undef SAVE_TIMEOUT
+
 static void
 hd_home_window_ensure_menu_status (HDHomeWindow *window)
 {
@@ -1030,7 +1375,7 @@ hd_home_window_ensure_menu_status (HDHomeWindow *window)
   GList                *items, *l;
 
   priv = HD_HOME_WINDOW_GET_PRIVATE (window);
-  area = hildon_home_window_get_area (HILDON_HOME_WINDOW (window));
+  area = GTK_BIN (window)->child;
 
   /* remove old children from the settings menu */
   items = gtk_container_get_children (GTK_CONTAINER (priv->settings_menu));
@@ -1060,7 +1405,7 @@ hd_home_window_ensure_menu_status (HDHomeWindow *window)
     {
       HildonDesktopHomeItem    *applet;
       GtkWidget                *item;
-  
+
       if (!HILDON_DESKTOP_IS_HOME_ITEM (l->data))
         continue;
       applet = HILDON_DESKTOP_HOME_ITEM (l->data);
@@ -1093,25 +1438,9 @@ hd_home_window_set_osso_context (HDHomeWindow *window,
 
   if (priv->osso_context != osso_context)
   {
-    GtkWidget *titlebar;
-
     priv->osso_context = osso_context;
     g_object_notify (G_OBJECT (window), "osso-context");
 
-    titlebar = hildon_home_window_get_titlebar (HILDON_HOME_WINDOW (window));
-
-    if (HILDON_IS_HOME_TITLEBAR (titlebar))
-    {
-#ifdef HAVE_LIBHILDONHELP
-      g_signal_connect_swapped (titlebar, "help-activate",
-                                G_CALLBACK (hd_home_window_help_activate),
-                                window);
-#endif
-      
-      g_signal_connect_swapped (titlebar, "applet-activate",
-                                G_CALLBACK (hd_home_window_applet_activate),
-                                window);
-    }
   }
 }
 #endif
@@ -1120,8 +1449,7 @@ static gboolean
 hd_home_window_destroy_banner (HDHomeWindow *window)
 {
   HDHomeWindowPrivate  *priv = HD_HOME_WINDOW_GET_PRIVATE (window);
-  GtkWidget            *area = hildon_home_window_get_area (
-                                    HILDON_HOME_WINDOW (window));
+  GtkWidget            *area = GTK_BIN (window)->child;
 
   if (priv->layout_mode_banner)
     {
@@ -1140,16 +1468,15 @@ static void
 hd_home_window_show_layout_mode_banner (HDHomeWindow *window)
 {
   HDHomeWindowPrivate  *priv = HD_HOME_WINDOW_GET_PRIVATE (window);
-  GtkWidget            *area = hildon_home_window_get_area (
-                                       HILDON_HOME_WINDOW (window));
-  
+  GtkWidget            *area = GTK_BIN (window)->child;
+
   if (!priv->layout_mode_banner)
     {
-      priv->layout_mode_banner = 
+      priv->layout_mode_banner =
           hildon_banner_show_animation (GTK_WIDGET (window),
                                         NULL,
                                         HH_LAYOUT_MODE_BANNER);
-      priv->layout_mode_banner_to = 
+      priv->layout_mode_banner_to =
           g_timeout_add (LAYOUT_OPENING_BANNER_TIMEOUT,
                          (GSourceFunc)hd_home_window_destroy_banner,
                          window);
@@ -1161,15 +1488,17 @@ hd_home_window_show_layout_mode_banner (HDHomeWindow *window)
 
 
     }
-    
+
 }
 
 static void
-hd_home_window_layout_mode_accept (HildonHomeWindow *window)
+hd_home_window_layout_mode_accept (HDHomeWindow *window)
 {
-  GtkWidget        *area;
+  HDHomeWindowPrivate  *priv = HD_HOME_WINDOW_GET_PRIVATE (window);
+  GtkWidget            *area;
 
-  area = hildon_home_window_get_area (window);
+  area = GTK_BIN (window)->child;
+  g_return_if_fail (HILDON_IS_HOME_AREA (area));
 
   if (hildon_home_area_get_overlaps (HILDON_HOME_AREA (area)))
     {
@@ -1178,17 +1507,25 @@ hd_home_window_layout_mode_accept (HildonHomeWindow *window)
       return;
     }
 
+  if (!priv->layout_mode_sucks)
+    {
+      hildon_home_area_set_layout_mode (HILDON_HOME_AREA (area),
+                                        FALSE);
+    }
+
+  hd_home_window_save_area_layout (HD_HOME_WINDOW (window));
+  g_signal_emit_by_name (window, "save", NULL);
 
-  if (HILDON_HOME_WINDOW_CLASS (hd_home_window_parent_class)->layout_mode_accept)
-    HILDON_HOME_WINDOW_CLASS (hd_home_window_parent_class)->layout_mode_accept (window);
 }
 
 static void
-hd_home_window_layout_mode_cancel (HildonHomeWindow *window)
+hd_home_window_layout_mode_cancel (HDHomeWindow *window)
 {
-  GtkWidget        *area;
+  HDHomeWindowPrivate  *priv = HD_HOME_WINDOW_GET_PRIVATE (window);
+  GtkWidget            *area;
 
-  area = hildon_home_window_get_area (window);
+  area = GTK_BIN (window)->child;
+  g_return_if_fail (HILDON_IS_HOME_AREA (area));
 
   if (hildon_home_area_get_layout_changed (HILDON_HOME_AREA (area)))
     {
@@ -1211,8 +1548,62 @@ hd_home_window_layout_mode_cancel (HildonHomeWindow *window)
         return;
     }
 
-  if (HILDON_HOME_WINDOW_CLASS (hd_home_window_parent_class)->layout_mode_cancel)
-    HILDON_HOME_WINDOW_CLASS (hd_home_window_parent_class)->layout_mode_cancel (window);
+  hd_home_window_load_area_layout (HD_HOME_WINDOW (window));
+  g_signal_emit_by_name (window, "load", NULL);
+
+  if (!priv->layout_mode_sucks)
+    hildon_home_area_set_layout_mode (HILDON_HOME_AREA (area),
+                                      FALSE);
+
+}
+
+static void
+hd_home_window_background (HDHomeWindow    *window,
+                           gboolean             is_background)
+{
+  HDHomeWindowPrivate  *priv = HD_HOME_WINDOW_GET_PRIVATE (window);
+  GtkWidget            *area;
+
+  area = GTK_BIN (window)->child;
+  g_return_if_fail (HILDON_IS_HOME_AREA (area));
+
+  if (!priv->is_inactive)
+    {
+
+      /* If we were in layout mode and went to background, we need
+       * to cancel it */
+      if (is_background &&
+          hildon_home_area_get_layout_mode (HILDON_HOME_AREA (area)))
+        {
+          hildon_home_area_set_layout_mode (HILDON_HOME_AREA (area),
+                                            FALSE);
+
+          hd_home_window_save_area_layout (window);
+        }
+
+      gtk_container_foreach (GTK_CONTAINER (area),
+                             (GtkCallback)hildon_desktop_home_item_set_is_background,
+                             (gpointer)is_background);
+    }
+
+}
+
+static void
+hd_home_window_system_inactivity (HDHomeWindow         *window,
+                                  gboolean              is_inactive)
+{
+  HDHomeWindowPrivate  *priv = HD_HOME_WINDOW_GET_PRIVATE (window);
+  GtkWidget            *area;
+
+  area = GTK_BIN (window)->child;
+  g_return_if_fail (HILDON_IS_HOME_AREA (area));
+
+  priv->is_inactive = is_inactive;
+
+  gtk_container_foreach (GTK_CONTAINER (area),
+                         (GtkCallback)hildon_desktop_home_item_set_is_background,
+                         (gpointer)is_inactive);
+
 }
 
 static void
@@ -1244,6 +1635,43 @@ hd_home_window_set_background (HDHomeWindow *window,
 
 }
 
+static void
+hd_home_window_select_applets (HDHomeWindow *window)
+{
+  HDHomeWindowPrivate  *priv = HD_HOME_WINDOW_GET_PRIVATE (window);
+  GtkWidget            *area = GTK_BIN (window)->child;
+
+  g_return_if_fail (HILDON_IS_HOME_AREA (area));
+
+  priv->selecting_applets = TRUE;
+
+  hildon_home_area_set_batch_add (HILDON_HOME_AREA (area), TRUE);
+  g_signal_emit_by_name (window, "select-plugins", NULL);
+  hildon_home_area_set_batch_add (HILDON_HOME_AREA (area), FALSE);
+  priv->selecting_applets = FALSE;
+
+  if (HILDON_IS_HOME_AREA (area))
+    {
+      /* If we are in layout mode, we only save when layout mode is
+       * accepted */
+      if (!hildon_home_area_get_layout_mode (HILDON_HOME_AREA (area)))
+        g_signal_emit_by_name (window, "save", NULL);
+    }
+
+}
+
+static void
+hd_home_window_cancel_layout (HDHomeWindow *window)
+{
+  g_signal_emit_by_name (window, "layout-mode-cancel");
+}
+
+static void
+hd_home_window_accept_layout (HDHomeWindow *window)
+{
+  g_signal_emit_by_name (window, "layout-mode-accept");
+}
+
 static void
 hd_home_window_set_background_reponse (HDHomeWindow *window,
                                        gint response,
@@ -1301,6 +1729,18 @@ hd_home_window_set_background_reponse (HDHomeWindow *window,
     }
 }
 
+static void
+hd_home_window_layout_mode_activate (HDHomeWindow *window)
+{
+  GtkWidget    *area = GTK_BIN (window)->child; 
+
+  g_return_if_fail (HILDON_IS_HOME_AREA (area));
+
+  hildon_home_area_set_layout_mode (HILDON_HOME_AREA (area),
+                                    TRUE);
+
+}
+
 static void
 hd_home_window_set_background_activate (HDHomeWindow *window)
 {
@@ -1313,8 +1753,8 @@ hd_home_window_set_background_activate (HDHomeWindow *window)
 #ifdef HAVE_LIBOSSO
                          "osso-context", priv->osso_context,
 #endif
-                         "background", priv->background,
                          "background-dir", HD_DESKTOP_BACKGROUNDS_PATH,
+                         "background", priv->background,
                          NULL);
 
   g_signal_connect_swapped (dialog, "response",
@@ -1332,9 +1772,9 @@ hd_home_window_show_information_note (HDHomeWindow *window,
   GtkWidget *note = NULL;
 
   note = hildon_note_new_information (NULL, text);
-		    
+
   gtk_dialog_run (GTK_DIALOG (note));
-  if (note) 
+  if (note)
     gtk_widget_destroy (GTK_WIDGET (note));
 }
 
